<?php
/**
 * XSLT Template Engine
 * 
 * @author Marcel Maa <info@mm-newmedia.de>
 * @copyright 2011 MM Newmedia <http://www.mm-newmedia.de>
 * @license GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @version 0.8
 */
class XSLTemplateEngine {
	
	/**
	 * DomDocument Objekt
	 * @var DomDocument
	 */
	protected $xml = null;
	
	/**
	 * Name of the XSL file
	 * @var string
	 */
	protected $xslFile = null;
	
	/**
	 * Root Element
	 * @var DomElement
	 */
	protected $root = null;
	
	/**
	 * creates a new XSLTemplateEngine
	 * 
	 * @param string $xslFile	the xsl file to use
	 * @param string $rootNode	name of the root element
	 * @param string $version	version of the xml document
	 * @param string $encoding	encoding of the xml document
	 */
	public function __construct($xslFile, $rootNode = 'root', $version = '1.0', $encoding = 'UTF-8') {
		$this->xslFile = $xslFile;
		$this->xml = new DOMDocument($version, $encoding);
	}
	
	/**
	 * assigns data to the XML document
	 * 
	 * @param array $data
	 */
	public function assignVariables($data) {
		$iterator = new RecursiveArrayIterator($data);
		iterator_apply($iterator, array($this, 'generateXmlNodes'), array($iterator, $this->xml));
	}
	
	/**
	 * recursive iterator for creating the xml document
	 * 
	 * @param RecursiveArrayIterator 	$data		Array with data to assign
	 * @param mixed 					$parentNode	Parent element to append the given data must be instance of DomDocument or DomElement
	 */
	private function generateXmlNodes(RecursiveArrayIterator $iterator, $parentNode) {
		while ($iterator->valid()) {
			if ($iterator->hasChildren()) {
				if (is_numeric($iterator->key())) {
					$node = $this->xml->createElement('item');
				} else {
					$node = $this->xml->createElement($iterator->key());
				}
				
				$node = $parentNode->appendChild($node);
				$this->generateXmlNodes(new RecursiveArrayIterator($iterator->current()), $node);
			} else {
				$node = $this->xml->createElement($iterator->key(), $iterator->current());
				$node = $parentNode->appendChild($node);
			}
			$iterator->next();
		}
	}
	
	/**
	 * loads an existing xml file
	 *
	 * @param string $filename name of the xml file
	 */
	public function assignFile($filename) {
		if (!file_exists($filename)) {
			throw new ErrorException('the given xml file does not exist', 10500);
		}
		$this->xml->load($filename);
	}
	
	/**
	 * outputs the XML document
	 */
	public function output() {
		$xslt = new XSLTProcessor();
		$xslt->registerPHPFunctions();
		
		$xsl = new DomDocument();
		$xsl->load($this->xslFile);
		$xslt->importStylesheet($xsl);
		
		echo $xslt->transformToXML($this->xml);
	}
}